package org.apache.ojb.broker.accesslayer;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.ojb.broker.metadata.JdbcConnectionDescriptor;
import org.apache.ojb.broker.util.logging.Logger;
import org.apache.ojb.broker.util.logging.LoggerFactory;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * Base implementation without connection pooling.
 *
 * @author <a href="mailto:armin@codeAuLait.de">Armin Waibel</a>
 * @version $Id: ConnectionFactoryNotPooledImpl.java,v 1.5 2004/04/04 23:53:30 brianm Exp $
 */
public class ConnectionFactoryNotPooledImpl extends ConnectionFactoryAbstractImpl
{
    private Logger log = LoggerFactory.getLogger(ConnectionFactoryNotPooledImpl.class);

    public Connection getConnectionFromPool(JdbcConnectionDescriptor jcd) throws LookupException
    {
        if (log.isDebugEnabled())
            log.debug(
                    "getConnectionFromPool was called, this implementation ever return a new connection");
        return this.newConnectionFromDriverManager(jcd);
    }

    public void returnConnectionToPool(JdbcConnectionDescriptor jcd, Connection con)
            throws LookupException
    {
        try
        {
            con.close();
        }
        catch (SQLException e)
        {
            log.warn("Connection.close() failed, message was " + e.getMessage());
        }
    }
}
