package org.apache.ojb.odmg;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.ojb.broker.util.logging.Logger;
import org.apache.ojb.broker.util.logging.LoggerFactory;
import org.odmg.Transaction;

import java.util.Hashtable;

/**
 * TransactionTable provides a mapping between the calling
 * thread and the Transaction it is currently using.
 * One thread can be joined with one transaction at
 * a certain point in time. But a thread can join with
 * different transactions subsequently.
 * This mapping from threads to Transactions is based on ODMG.
 *
 * @author     Thomas Mahler & David Dixon-Peugh
 * @version $Id: TransactionTable.java,v 1.6 2004/04/04 23:53:39 brianm Exp $
 */
public class TransactionTable extends Object
{
    private Logger log = LoggerFactory.getLogger(TransactionTable.class);

    /**
     * the internal Hashtable mapping Transactions to threads
     */
    private Hashtable m_table = new Hashtable();

    /**
     * Creates new TransactionTable
     */
    public TransactionTable()
    {
    }

    /**
     * Retreive a Transaction associated with a thread.
     *
     * @param  key_thread  The thread to lookup.
     * @return             The transaction associated with the thread.
     */
    public Transaction get(Thread key_thread)
    {
        return (Transaction) m_table.get(key_thread);
    }

    /**
     * Store the Thread/Transaction pair in the TransactionTable
     *
     * @param  key_thread  Thread that the transaction will be associated to
     * @param  value_tx    Transaction to be associated with the thread
     */
    public void put(Thread key_thread, Transaction value_tx)
    {
        m_table.put(key_thread, value_tx);
    }

    /**
     * Remove the entry for the thread
     * @param  key_thread  Thread to be removed.
     */
    public void remove(Thread key_thread)
    {
        m_table.remove(key_thread);
    }
}
