package org.apache.ojb.odmg.collections;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Iterator;

import org.apache.ojb.odmg.TransactionImpl;
import org.apache.ojb.odmg.TxManagerFactory;

/**
 * Insert the type's description here.
 * Creation date: (26.02.2001 13:18:55)
 * @author Thomas Mahler
 */
public class DSetIterator implements Iterator
{
    private Iterator iter;
    private DSetEntry currentEntry = null;

    /**
     * DListIterator constructor comment.
     */
    public DSetIterator(DSetImpl set)
    {
        this.iter = set.getElements().iterator();
    }

    /**
     * Returns <tt>true</tt> if the iteration has more elements. (In other
     * words, returns <tt>true</tt> if <tt>next</tt> would return an element
     * rather than throwing an exception.)
     *
     * @return <tt>true</tt> if the iterator has more elements.
     */
    public boolean hasNext()
    {
        return iter.hasNext();
    }

    /**
     * Returns the next element in the interation.
     *
     * @return the next element in the interation.
     * @exception NoSuchElementException iteration has no more elements.
     */
    public Object next()
    {
        currentEntry = ((DSetEntry) iter.next());
        return currentEntry.getRealSubject();
    }

    /**
     *
     * Removes from the underlying collection the last element returned by the
     * iterator (optional operation).  This method can be called only once per
     * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
     * the underlying collection is modified while the iteration is in
     * progress in any way other than by calling this method.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * operation is not supported by this Iterator.
     *
     * @exception IllegalStateException if the <tt>next</tt> method has not
     * yet been called, or the <tt>remove</tt> method has already
     * been called after the last call to the <tt>next</tt>
     * method.
     */
    public void remove()
    {
        iter.remove();
        TransactionImpl tx = TxManagerFactory.instance().getTransaction();
        if (tx != null)
        {
            tx.markDelete(currentEntry);
        }
        currentEntry = null;
    }

}
